TRAFFIC_WRITE_KEY = 'a9f1566e48a366068bf933467noreal'      # <----  Use your own hereT
THREE_BODY_WRITE_KEY = '3efc6d1e0c1c17603486f2d16notreal'  # <----  Use your own here
BASE_URL = 'https://www.microprediction.org'
